@@ -1,65 +1,105 @@
 from django.db import models
-from django.core.urlresolvers import reverse
-from django.utils.text import slugify
 from django.db.models import Sum
 from hurry.filesize import size
+from django.utils.text import slugify
+from django.core.urlresolvers import reverse
 
 
 class Filer(models.Model):
+    """
+    An entity that files campaign finance disclosure documents.
+
+    That includes candidates for public office that have committees raising
+    money on their behalf (i.e. Jerry Brown) as well as Political Action
+    Committees (PACs) that contribute money to numerous candidates for office.
+    """
+    # straight out of the filer table
+    filer_id = models.IntegerField(db_index=True)
+    STATUS_CHOICES = (
+        ('A', 'A'),
+        ('ACTIVE', 'Active'),
+        ('INACTIVE', 'Inactive'),
+        ('R', 'R'),
+        ('S', 'S'),
+        ('TERMINATED', 'Terminated'),
+        ('W', 'W'),
+    )
+    status = models.CharField(
+        max_length=255,
+        null=True,
+        choices=STATUS_CHOICES
+    )
     FILER_TYPE_OPTIONS = (
-        ('pac', 'Political Action Committee'),
+        ('pac', 'PAC'),
         ('cand', 'Candidate'),
     )
-    # straight out of the filer table
-    filer_id = models.IntegerField()
-    status = models.CharField(max_length=255, null=True)
-    filer_type = models.CharField(max_length=10L, choices=FILER_TYPE_OPTIONS)
+    filer_type = models.CharField(
+        max_length=10,
+        choices=FILER_TYPE_OPTIONS,
+        db_index=True,
+    )
+    name = models.CharField(max_length=255, null=True)
     effective_date = models.DateField(null=True)
-    # fields updated by other tables
-    xref_filer_id = models.CharField(max_length=32L, null=True)
-    name = models.CharField(max_length=255L, null=True)
+    xref_filer_id = models.CharField(
+        max_length=32,
+        null=True,
+        db_index=True
+    )
+
+    class Meta:
+        ordering = ("name",)
 
     def __unicode__(self):
+        return self.short_name
+
+    @models.permalink
+    def get_absolute_url(self):
+        return ('filer_detail', [str(self.pk)])
+
+    @property
+    def short_name(self, character_limit=75):
+        if len(self.name) > character_limit:
+            return self.name[:character_limit] + "..."
         return self.name
 
-    def get_absolute_url(self):
-        return reverse('filer_detail', args=[str(self.pk)])
-
-    def _create_slug(self):
+    @property
+    def slug(self):
         return slugify(self.name)
 
-    slug = property(_create_slug)
-
-    def _total_contributions(self):
-
+    @property
+    def total_contributions(self):
         qs = Filing.objects.filter(committee__filer=self)
         total = Summary.objects.filter(filing__in=qs).aggregate(
-            tot=Sum('total_contribs'))['tot']
+            tot=Sum('total_contributions'))['tot']
         return total
 
-    total_contributions = property(_total_contributions)
-
 
 class Committee(models.Model):
-
-    '''
-        If a Candidate controls the committee, the filer is associated with the
-        Candidate Filer record, not the committee Filer record
-        But the committee Filer record can still be accessed using filer_id_raw
-        So candidate filers potentially link to multiple committes,
-        and committee filers that are not candidate controlled
-        link back to one, committee filer
-        If there's a better way I'm open to suggestions
-    '''
+    """
+    If a Candidate controls the committee, the filer is associated with the
+    Candidate Filer record, not the committee Filer record
+    But the committee Filer record can still be accessed using filer_id_raw
+    So candidate filers potentially link to multiple committes,
+    and committee filers that are not candidate controlled
+    link back to one, committee filer
+    If there's a better way I'm open to suggestions
+    """
+    filer = models.ForeignKey(Filer)
+    filer_id_raw = models.IntegerField(db_index=True)
+    name = models.CharField(max_length=255, null=True)
     CMTE_TYPE_OPTIONS = (
-        ('cand', 'Candidate Committee'),
-        ('pac', 'Non-Candidate Committee'),
+        ('cand', 'Candidate'),
+        ('pac', 'PAC'),
         ('linked-pac', 'Non-Candidate Committee, linked to other committees'),
     )
-    filer = models.ForeignKey(Filer)
-    filer_id_raw = models.IntegerField()
-    name = models.CharField(max_length=255, null=True)
-    committee_type = models.CharField(max_length=50, choices=CMTE_TYPE_OPTIONS)
+    committee_type = models.CharField(
+        max_length=50,
+        choices=CMTE_TYPE_OPTIONS,
+        db_index=True,
+    )
+
+    class Meta:
+        ordering = ("name",)
 
     def __unicode__(self):
         return self.name
@@ -67,149 +107,185 @@ class Committee(models.Model):
     def get_absolute_url(self):
         return reverse('committee_detail', args=[str(self.pk)])
 
-    def _total_contributions(self):
+    @property
+    def short_name(self, character_limit=75):
+        if len(self.name) > character_limit:
+            return self.name[:character_limit] + "..."
+        return self.name
+
+    @property
+    def total_contributions(self):
         qs = Filing.objects.filter(committee=self)
         total = Summary.objects.filter(filing__in=qs).aggregate(
-            tot=Sum('total_contribs'))['tot']
+            tot=Sum('total_contributions'))['tot']
         return total
 
-    total_contributions = property(_total_contributions)
-
-    def _total_expenditures(self):
+    @property
+    def total_expenditures(self):
         qs = Filing.objects.filter(committee=self)
         total = Summary.objects.filter(filing__in=qs).aggregate(
             tot=Sum('total_expenditures'))['tot']
         return total
 
-    total_expenditures = property(_total_expenditures)
-
     def links(self):
+        from calaccess_raw.models import FilerLinksCd, FilernameCd, LookupCode
         d = {}
-        try:
-            from calaccess_raw.models import (
-                FilerLinksCd, FilernameCd, LookupCode
-            )
-            qs_links = FilerLinksCd.objects.filter(
-                filer_id_a=self.filer_id_raw)
-            for q in qs_links:
-                qs_names = (
-                    FilernameCd
-                    .objects
-                    .filter(filer_id=q.filer_id_b)
-                    .order_by('-effect_dt')
-                    .exclude(naml=''))
+        qs_links = FilerLinksCd.objects.filter(
+            filer_id_a=self.filer_id_raw)
+        for q in qs_links:
+            qs_names = (
+                FilernameCd
+                .objects
+                .filter(filer_id=q.filer_id_b)
+                .order_by('-effect_dt')
+                .exclude(naml=''))
 
-                if qs_names > 0:
-                    # committee filer name object
-                    obj = qs_names[0]
-                    name = (
-                        "{0} {1} {2} {3}"
-                        .format(obj.namt, obj.namf, obj.naml, obj.nams)
-                        .strip()
-                    )
-
-                else:
-                    name = ''
-
-                d[q.filer_id_b] = {
-                    'link_type': (
-                        LookupCode
-                        .objects
-                        .get(code_id=q.link_type)
-                        .code_desc
-                    ),
-                    'filer_id_b': q.filer_id_b,
-                    'filer_name': name,
-                    'effective_date': q.effect_dt,
-                }
-
-        except:
-            print 'Raw data not available. Install and run calaccess_raw app.'
-
-        return d
-
-    def print_links(self):
-        d = self.links()
-        if d:
-            for v in d.values():
-                print (
-                    '{0}\t{1}\t{2}\t{3}'
-                    .format(
-                        v['filer_id_b'],
-                        v['link_type'],
-                        v['effective_date'],
-                        v['filer_name']
-                    )
+            if qs_names > 0:
+                # committee filer name object
+                obj = qs_names[0]
+                name = (
+                    "{0} {1} {2} {3}"
+                    .format(obj.namt, obj.namf, obj.naml, obj.nams)
+                    .strip()
                 )
 
+            else:
+                name = ''
+
+            d[q.filer_id_b] = {
+                'link_type': (
+                    LookupCode
+                    .objects
+                    .get(code_id=q.link_type)
+                    .code_desc
+                ),
+                'filer_id_b': q.filer_id_b,
+                'filer_name': name,
+                'effective_date': q.effect_dt,
+            }
+        return d
+
 
 class Cycle(models.Model):
-    name = models.IntegerField()
+    name = models.IntegerField(db_index=True)
+
+    class Meta:
+        ordering = ("-name",)
 
     def __unicode__(self):
-        str_name = '%s' % self.name
-        return str_name
+        return self.name
 
 
 class Filing(models.Model):
     cycle = models.ForeignKey(Cycle)
     committee = models.ForeignKey(Committee)
-    filing_id_raw = models.IntegerField()
-    amend_id = models.IntegerField()
-    form_id = models.CharField(max_length=7)
+    filing_id_raw = models.IntegerField(db_index=True)
+    amend_id = models.IntegerField(db_index=True)
+    form_id = models.CharField(max_length=7, db_index=True)
     start_date = models.DateField(null=True)
     end_date = models.DateField(null=True)
-    dupe = models.BooleanField(default=False)
+    date_received = models.DateField(null=True)
+    date_filed = models.DateField(null=True)
+    dupe = models.BooleanField(
+        default=False,
+        db_index=True,
+        verbose_name='duplicate',
+    )
 
     def __unicode__(self):
-        str_name = '%s (%s - %s)' % (self.filing_id_raw,
-                                     self.start_date, self.end_date)
-        return str_name
+        return '%s (%s-%s)' % (
+            self.filing_id_raw,
+            self.start_date,
+            self.end_date
+        )
 
     def get_absolute_url(self):
         return reverse('filing_detail', args=[str(self.pk)])
 
 
 class Summary(models.Model):
+    cycle = models.ForeignKey(Cycle)
+    committee = models.ForeignKey(Committee)
+    filing = models.ForeignKey(Filing)
     FORM_TYPE_CHOICES = (
         ('F460', 'Recipient Committee Campaign Statement'),
         ('F450', 'Recipient Committee Campaign Statement -- Short Form'),
     )
-    cycle = models.ForeignKey(Cycle)
-    committee = models.ForeignKey(Committee)
-    filing = models.ForeignKey(Filing)
-    form_type = models.CharField(max_length=10, choices=FORM_TYPE_CHOICES)
-    itemized_monetary_contribs = models.DecimalField(
-        max_digits=16, decimal_places=2)
+    form_type = models.CharField(
+        max_length=10,
+        choices=FORM_TYPE_CHOICES,
+        db_index=True,
+    )
+    dupe = models.BooleanField(default=False, db_index=True)
+    itemized_monetary_contributions = models.DecimalField(
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
     unitemized_expenditures = models.DecimalField(
-        max_digits=16, decimal_places=2)
-    total_expenditures = models.DecimalField(max_digits=16, decimal_places=2)
-    total_monetary_contribs = models.DecimalField(
-        max_digits=16, decimal_places=2)
-    unitemized_monetary_contribs = models.DecimalField(
-        max_digits=16, decimal_places=2)
-    non_monetary_contribs = models.DecimalField(
-        max_digits=16, decimal_places=2)
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
+    total_expenditures = models.DecimalField(
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
+    total_monetary_contributions = models.DecimalField(
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
+    unitemized_monetary_contributions = models.DecimalField(
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
+    non_monetary_contributions = models.DecimalField(
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
     itemized_expenditures = models.DecimalField(
-        max_digits=16, decimal_places=2)
-    total_contribs = models.DecimalField(max_digits=16, decimal_places=2)
-    outstanding_debts = models.DecimalField(max_digits=16, decimal_places=2)
-    ending_cash_balance = models.DecimalField(max_digits=16, decimal_places=2)
-    dupe = models.BooleanField(default=False)
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
+    total_contributions = models.DecimalField(
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
+    outstanding_debts = models.DecimalField(
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
+    ending_cash_balance = models.DecimalField(
+        max_digits=16,
+        decimal_places=2,
+        null=True,
+        default=None,
+    )
 
     def __unicode__(self):
-        str_name = (
-            '{0} {1} ({2} - {3})'
-            .format(
-                self.cycle.name,
-                self.committee.name,
-                self.filing.start_date,
-                self.filing.end_date
-            )
+        return '{0} {1} ({2} - {3})'.format(
+            self.cycle.name,
+            self.committee.name,
+            self.filing.start_date,
+            self.filing.end_date
         )
 
-        return str_name
-
 
 class Expenditure(models.Model):
 
@@ -267,7 +343,6 @@ class Expenditure(models.Model):
         max_length=3L, blank=True,
         choices=EXPENDITURE_CODE_CHOICES
     )
-    expn_code_display = models.CharField(max_length=255, blank=True)
     expn_date = models.DateField(null=True)
     expn_dscr = models.CharField(max_length=400L, blank=True)
     form_type = models.CharField(max_length=6L, blank=True)
@@ -301,23 +376,15 @@ class Expenditure(models.Model):
 
     dupe = models.BooleanField(default=False)
 
+    @property
     def raw(self):
-        try:
-            from calaccess_raw.models import ExpnCd
-            obj = ExpnCd.objects.get(
-                amend_id=self.amend_id,
-                filing_id=self.filing_id,
-                tran_id=self.tran_id,
-                bakref_tid=self.bakref_tid
-            )
-        except:
-            print 'Raw data not available. Install and run calaccess_raw app.'
-            obj = None
-        return obj
-
-    def save(self, **kwargs):
-        self.expn_code_display = self.get_expn_code_display()
-        super(self.__class__, self).save(**kwargs)
+        from calaccess_raw.models import ExpnCd
+        return ExpnCd.objects.get(
+            amend_id=self.amend_id,
+            filing_id=self.filing_id,
+            tran_id=self.tran_id,
+            bakref_tid=self.bakref_tid
+        )
 
     def get_absolute_url(self):
         return reverse('expenditure_detail', args=[str(self.pk)])
@@ -383,27 +450,21 @@ class Contribution(models.Model):
     individual_id = models.IntegerField(null=True)
     dupe = models.BooleanField(default=False)
 
+    @property
     def raw(self):
-        try:
-            from calaccess_raw.models import RcptCd
-            obj = RcptCd.objects.get(
-                amend_id=self.amend_id,
-                filing_id=self.filing_id,
-                tran_id=self.tran_id,
-                bakref_tid=self.bakref_tid
-            )
-
-        except:
-            print 'Raw data not available. Install and run calaccess app.'
-            obj = None
-        return obj
+        from calaccess_raw.models import RcptCd
+        return RcptCd.objects.get(
+            amend_id=self.amend_id,
+            filing_id=self.filing_id,
+            tran_id=self.tran_id,
+            bakref_tid=self.bakref_tid
+        )
 
     def get_absolute_url(self):
         return reverse('contribution_detail', args=[str(self.pk)])
 
 
 class Stats(models.Model):
-
     '''
         Flexible model for housing aggregate stats
         Should be able to add any stat you like for
@@ -417,9 +478,12 @@ class Stats(models.Model):
         ('pac', 'Political Action Committee'),
     )
     STAT_TYPE_CHOICES = (
-        ('itemized_monetary_contribs', 'Itemized Monetary Contributions'),
-        ('unitemized_monetary_contribs', 'Unitemized Monetary Contributions'),
-        ('total_contribs', 'Total Contributions'),
+        ('itemized_monetary_contributions', 'Itemized Monetary Contributions'),
+        (
+            'unitemized_monetary_contributions',
+            'Unitemized Monetary Contributions'
+        ),
+        ('total_contributions', 'Total Contributions'),
         ('total_expenditures', 'Total Expenditures'),
         ('outstanding_debts', 'Outstanding Debt'),
 
@@ -434,8 +498,7 @@ class Stats(models.Model):
     amount = models.DecimalField(max_digits=16, decimal_places=2)
 
     def __unicode__(self):
-        name_str = '%s-%s' % (self.filer_type, self.stat_type)
-        return name_str
+        return '%s-%s' % (self.filer_type, self.stat_type)
 
 
 class FlatFile(models.Model):
